@@ -688,7 +688,6 @@ local_binding_add_child(struct local_binding *lbinding,
                         struct local_binding *child)
 {
     local_binding_add(&lbinding->children, child);
-    child->parent = lbinding;
 }
 
 static struct local_binding *
@@ -698,13 +697,6 @@ local_binding_find_child(struct local_binding *lbinding,
     return local_binding_find(&lbinding->children, child_name);
 }
 
-static void
-local_binding_delete_child(struct local_binding *lbinding,
-                           struct local_binding *child)
-{
-    shash_find_and_delete(&lbinding->children, child->name);
-}
-
 static bool
 is_lport_vif(const struct sbrec_port_binding *pb)
 {
@@ -966,7 +958,8 @@ release_local_binding_children(const struct sbrec_chassis *chassis_rec,
             }
         }
 
-        /* Clear the local bindings' 'iface'. */
+        /* Clear the local bindings' 'pb' and 'iface'. */
+        l->pb = NULL;
         l->iface = NULL;
     }
 
@@ -1530,41 +1523,21 @@ binding_cleanup(struct ovsdb_idl_txn *ovnsb_idl_txn,
 }
 
 static const struct sbrec_port_binding *
-get_peer_lport__(const struct sbrec_port_binding *pb,
-                 struct binding_ctx_in *b_ctx_in)
+get_peer_lport(const struct sbrec_port_binding *pb,
+               struct binding_ctx_in *b_ctx_in)
 {
     const char *peer_name = smap_get(&pb->options, "peer");
-
-    if (!peer_name) {
+    if (strcmp(pb->type, "patch") || !peer_name) {
         return NULL;
     }
 
     const struct sbrec_port_binding *peer;
     peer = lport_lookup_by_name(b_ctx_in->sbrec_port_binding_by_name,
                                 peer_name);
+
     return (peer && peer->datapath) ? peer : NULL;
 }
 
-static const struct sbrec_port_binding *
-get_l3gw_peer_lport(const struct sbrec_port_binding *pb,
-                    struct binding_ctx_in *b_ctx_in)
-{
-    if (strcmp(pb->type, "l3gateway")) {
-        return NULL;
-    }
-    return get_peer_lport__(pb, b_ctx_in);
-}
-
-static const struct sbrec_port_binding *
-get_peer_lport(const struct sbrec_port_binding *pb,
-               struct binding_ctx_in *b_ctx_in)
-{
-    if (strcmp(pb->type, "patch")) {
-        return NULL;
-    }
-    return get_peer_lport__(pb, b_ctx_in);
-}
-
 /* This function adds the local datapath of the 'peer' of
  * lport 'pb' to the local datapaths if it is not yet added.
  */
@@ -1681,9 +1654,7 @@ remove_pb_from_local_datapath(const struct sbrec_port_binding *pb,
                                          pb->logical_port)) {
             ld->localnet_port = NULL;
         }
-    }
-
-    if (!strcmp(pb->type, "l3gateway")) {
+    } else if (!strcmp(pb->type, "l3gateway")) {
         const char *chassis_id = smap_get(&pb->options,
                                           "l3gateway-chassis");
         if (chassis_id && !strcmp(chassis_id, chassis_rec->name)) {
@@ -1985,27 +1956,12 @@ handle_deleted_lport(const struct sbrec_port_binding *pb,
                      struct binding_ctx_in *b_ctx_in,
                      struct binding_ctx_out *b_ctx_out)
 {
-    /* If the binding is local, remove it. */
     struct local_datapath *ld =
         get_local_datapath(b_ctx_out->local_datapaths,
                            pb->datapath->tunnel_key);
     if (ld) {
         remove_pb_from_local_datapath(pb, b_ctx_in->chassis_rec,
                                       b_ctx_out, ld);
-        return;
-    }
-
-    /* If the binding is not local, if 'pb' is a L3 gateway port, we should
-     * remove its peer, if that one is local.
-     */
-    pb = get_l3gw_peer_lport(pb, b_ctx_in);
-    if (pb) {
-        ld = get_local_datapath(b_ctx_out->local_datapaths,
-                                pb->datapath->tunnel_key);
-        if (ld) {
-            remove_pb_from_local_datapath(pb, b_ctx_in->chassis_rec, b_ctx_out,
-                                          ld);
-        }
     }
 }
 
@@ -2070,16 +2026,6 @@ handle_deleted_vif_lport(const struct sbrec_port_binding *pb,
      * when the interface change happens. */
     if (is_lport_container(pb)) {
         remove_local_lports(pb->logical_port, b_ctx_out);
-
-        /* If the container port is removed we should also remove it from
-         * its parent's children set.
-         */
-        if (lbinding) {
-            if (lbinding->parent) {
-                local_binding_delete_child(lbinding->parent, lbinding);
-            }
-            local_binding_destroy(lbinding);
-        }
     }
 
     handle_deleted_lport(pb, b_ctx_in, b_ctx_out);
@@ -2149,26 +2095,13 @@ bool
 binding_handle_port_binding_changes(struct binding_ctx_in *b_ctx_in,
                                     struct binding_ctx_out *b_ctx_out)
 {
-    /* Run the tracked port binding loop twice to ensure correctness:
-     * 1. First to handle deleted changes.  This is split in four sub-parts
-     *    because child local bindings must be cleaned up first:
-     *    a. Container ports first.
-     *    b. Then virtual ports.
-     *    c. Then regular VIFs.
-     *    d. Last other ports.
-     * 2. Second to handle add/update changes.
-     */
-    struct shash deleted_container_pbs =
-        SHASH_INITIALIZER(&deleted_container_pbs);
-    struct shash deleted_virtual_pbs =
-        SHASH_INITIALIZER(&deleted_virtual_pbs);
-    struct shash deleted_vif_pbs =
-        SHASH_INITIALIZER(&deleted_vif_pbs);
-    struct shash deleted_other_pbs =
-        SHASH_INITIALIZER(&deleted_other_pbs);
-    const struct sbrec_port_binding *pb;
     bool handled = true;
+    const struct sbrec_port_binding *pb;
 
+    /* Run the tracked port binding loop twice. One to handle deleted
+     * changes. And another to handle add/update changes.
+     * This will ensure correctness.
+     */
     SBREC_PORT_BINDING_TABLE_FOR_EACH_TRACKED (pb,
                                                b_ctx_in->port_binding_table) {
         if (!sbrec_port_binding_is_deleted(pb)) {
@@ -2176,60 +2109,18 @@ binding_handle_port_binding_changes(struct binding_ctx_in *b_ctx_in,
         }
 
         enum en_lport_type lport_type = get_lport_type(pb);
-
-        if (lport_type == LP_VIF) {
-            if (is_lport_container(pb)) {
-                shash_add(&deleted_container_pbs, pb->logical_port, pb);
-            } else {
-                shash_add(&deleted_vif_pbs, pb->logical_port, pb);
-            }
-        } else if (lport_type == LP_VIRTUAL) {
-            shash_add(&deleted_virtual_pbs, pb->logical_port, pb);
+        if (lport_type == LP_VIF || lport_type == LP_VIRTUAL) {
+            handled = handle_deleted_vif_lport(pb, lport_type, b_ctx_in,
+                                                b_ctx_out);
         } else {
-            shash_add(&deleted_other_pbs, pb->logical_port, pb);
+            handle_deleted_lport(pb, b_ctx_in, b_ctx_out);
         }
-    }
 
-    struct shash_node *node;
-    struct shash_node *node_next;
-    SHASH_FOR_EACH_SAFE (node, node_next, &deleted_container_pbs) {
-        handled = handle_deleted_vif_lport(node->data, LP_VIF, b_ctx_in,
-                                           b_ctx_out);
-        shash_delete(&deleted_container_pbs, node);
         if (!handled) {
-            goto delete_done;
+            break;
         }
     }
 
-    SHASH_FOR_EACH_SAFE (node, node_next, &deleted_virtual_pbs) {
-        handled = handle_deleted_vif_lport(node->data, LP_VIRTUAL, b_ctx_in,
-                                           b_ctx_out);
-        shash_delete(&deleted_virtual_pbs, node);
-        if (!handled) {
-            goto delete_done;
-        }
-    }
-
-    SHASH_FOR_EACH_SAFE (node, node_next, &deleted_vif_pbs) {
-        handled = handle_deleted_vif_lport(node->data, LP_VIF, b_ctx_in,
-                                           b_ctx_out);
-        shash_delete(&deleted_vif_pbs, node);
-        if (!handled) {
-            goto delete_done;
-        }
-    }
-
-    SHASH_FOR_EACH_SAFE (node, node_next, &deleted_other_pbs) {
-        handle_deleted_lport(node->data, b_ctx_in, b_ctx_out);
-        shash_delete(&deleted_other_pbs, node);
-    }
-
-delete_done:
-    shash_destroy(&deleted_container_pbs);
-    shash_destroy(&deleted_virtual_pbs);
-    shash_destroy(&deleted_vif_pbs);
-    shash_destroy(&deleted_other_pbs);
-
     if (!handled) {
         return false;
     }
