@@ -36,6 +36,44 @@ VLOG_DEFINE_THIS_MODULE(chassis);
 #define HOST_NAME_MAX 255
 #endif /* HOST_NAME_MAX */
 
+/*
+ * Structure to hold chassis specific state (currently just chassis-id)
+ * to avoid database lookups when changes happen while the controller is
+ * running.
+ */
+struct chassis_info {
+    /* Last ID we initialized the Chassis SB record with. */
+    struct ds id;
+
+    /* True if Chassis SB record is initialized, false otherwise. */
+    uint32_t id_inited : 1;
+};
+
+static struct chassis_info chassis_state = {
+    .id = DS_EMPTY_INITIALIZER,
+    .id_inited = false,
+};
+
+static void
+chassis_info_set_id(struct chassis_info *info, const char *id)
+{
+    ds_clear(&info->id);
+    ds_put_cstr(&info->id, id);
+    info->id_inited = true;
+}
+
+static bool
+chassis_info_id_inited(const struct chassis_info *info)
+{
+    return info->id_inited;
+}
+
+static const char *
+chassis_info_id(const struct chassis_info *info)
+{
+    return ds_cstr_ro(&info->id);
+}
+
 /*
  * Structure for storing the chassis config parsed from the ovs table.
  */
@@ -48,6 +86,7 @@ struct ovs_chassis_cfg {
     const char *cms_options;
     const char *monitor_all;
     const char *chassis_macs;
+    const char *enable_lflow_cache;
 
     /* Set of encap types parsed from the 'ovn-encap-type' external-id. */
     struct sset encap_type_set;
@@ -127,6 +166,12 @@ get_monitor_all(const struct smap *ext_ids)
     return smap_get_def(ext_ids, "ovn-monitor-all", "false");
 }
 
+static const char *
+get_enable_lflow_cache(const struct smap *ext_ids)
+{
+    return smap_get_def(ext_ids, "ovn-enable-lflow-cache", "true");
+}
+
 static const char *
 get_encap_csum(const struct smap *ext_ids)
 {
@@ -183,7 +228,8 @@ chassis_parse_ovs_encap_type(const char *encap_type,
 
     SSET_FOR_EACH (type, encap_type_set) {
         if (!get_tunnel_type(type)) {
-            VLOG_INFO("Unknown tunnel type: %s", type);
+            static struct vlog_rate_limit rl = VLOG_RATE_LIMIT_INIT(1, 5);
+            VLOG_INFO_RL(&rl, "Unknown tunnel type: %s", type);
         }
     }
 
@@ -235,7 +281,8 @@ chassis_parse_ovs_config(const struct ovsrec_open_vswitch_table *ovs_table,
     const char *encap_type = smap_get(&cfg->external_ids, "ovn-encap-type");
     const char *encap_ips = smap_get(&cfg->external_ids, "ovn-encap-ip");
     if (!encap_type || !encap_ips) {
-        VLOG_INFO("Need to specify an encap type and ip");
+        static struct vlog_rate_limit rl = VLOG_RATE_LIMIT_INIT(1, 5);
+        VLOG_INFO_RL(&rl, "Need to specify an encap type and ip");
         return false;
     }
 
@@ -246,6 +293,7 @@ chassis_parse_ovs_config(const struct ovsrec_open_vswitch_table *ovs_table,
     ovs_cfg->cms_options = get_cms_options(&cfg->external_ids);
     ovs_cfg->monitor_all = get_monitor_all(&cfg->external_ids);
     ovs_cfg->chassis_macs = get_chassis_mac_mappings(&cfg->external_ids);
+    ovs_cfg->enable_lflow_cache = get_enable_lflow_cache(&cfg->external_ids);
 
     if (!chassis_parse_ovs_encap_type(encap_type, &ovs_cfg->encap_type_set)) {
         return false;
@@ -272,12 +320,14 @@ static void
 chassis_build_other_config(struct smap *config, const char *bridge_mappings,
                            const char *datapath_type, const char *cms_options,
                            const char *monitor_all, const char *chassis_macs,
-                           const char *iface_types, bool is_interconn)
+                           const char *iface_types,
+                           const char *enable_lflow_cache, bool is_interconn)
 {
     smap_replace(config, "ovn-bridge-mappings", bridge_mappings);
     smap_replace(config, "datapath-type", datapath_type);
     smap_replace(config, "ovn-cms-options", cms_options);
     smap_replace(config, "ovn-monitor-all", monitor_all);
+    smap_replace(config, "ovn-enable-lflow-cache", enable_lflow_cache);
     smap_replace(config, "iface-types", iface_types);
     smap_replace(config, "ovn-chassis-mac-mappings", chassis_macs);
     smap_replace(config, "is-interconn", is_interconn ? "true" : "false");
@@ -292,6 +342,7 @@ chassis_other_config_changed(const char *bridge_mappings,
                              const char *cms_options,
                              const char *monitor_all,
                              const char *chassis_macs,
+                             const char *enable_lflow_cache,
                              const struct ds *iface_types,
                              bool is_interconn,
                              const struct sbrec_chassis *chassis_rec)
@@ -324,6 +375,13 @@ chassis_other_config_changed(const char *bridge_mappings,
         return true;
     }
 
+    const char *chassis_enable_lflow_cache =
+        get_enable_lflow_cache(&chassis_rec->other_config);
+
+    if (strcmp(enable_lflow_cache, chassis_enable_lflow_cache)) {
+        return true;
+    }
+
     const char *chassis_mac_mappings =
         get_chassis_mac_mappings(&chassis_rec->other_config);
     if (strcmp(chassis_macs, chassis_mac_mappings)) {
@@ -362,9 +420,6 @@ chassis_tunnels_changed(const struct sset *encap_type_set,
     bool  changed = false;
 
     for (size_t i = 0; i < chassis_rec->n_encaps; i++) {
-        if (strcmp(chassis_rec->name, chassis_rec->encaps[i]->chassis_name)) {
-            return true;
-        }
 
         if (!sset_contains(encap_type_set, chassis_rec->encaps[i]->type)) {
             changed = true;
@@ -445,11 +500,54 @@ chassis_build_encaps(struct ovsdb_idl_txn *ovnsb_idl_txn,
     return encaps;
 }
 
+/*
+ * Updates encaps for a given chassis. This can happen when the chassis
+ * name has changed. Also, the only thing we support updating is the
+ * chassis_name. For other changes the encaps will be recreated.
+ */
+static void
+chassis_update_encaps(const struct sbrec_chassis *chassis)
+{
+    for (size_t i = 0; i < chassis->n_encaps; i++) {
+        sbrec_encap_set_chassis_name(chassis->encaps[i], chassis->name);
+    }
+}
+
+/*
+ * Returns a pointer to a chassis record from 'chassis_table' that
+ * matches at least one tunnel config.
+ */
+static const struct sbrec_chassis *
+chassis_get_stale_record(const struct sbrec_chassis_table *chassis_table,
+                         const struct ovs_chassis_cfg *ovs_cfg,
+                         const char *chassis_id)
+{
+    const struct sbrec_chassis *chassis_rec;
+
+    SBREC_CHASSIS_TABLE_FOR_EACH (chassis_rec, chassis_table) {
+        for (size_t i = 0; i < chassis_rec->n_encaps; i++) {
+            if (sset_contains(&ovs_cfg->encap_type_set,
+                              chassis_rec->encaps[i]->type) &&
+                    sset_contains(&ovs_cfg->encap_ip_set,
+                                  chassis_rec->encaps[i]->ip)) {
+                return chassis_rec;
+            }
+            if (strcmp(chassis_rec->name, chassis_id) == 0) {
+                return chassis_rec;
+            }
+        }
+    }
+
+    return NULL;
+}
+
 /* If this is a chassis config update after we initialized the record once
  * then we should always be able to find it with the ID we saved in
  * chassis_state.
  * Otherwise (i.e., first time we create the record or if the system-id
- * changed) we create a new record.
+ * changed) then we check if there's a stale record from a previous
+ * controller run that didn't end gracefully and reuse it. If not then we
+ * create a new record.
  *
  * Sets '*chassis_rec' to point to the local chassis record.
  * Returns true if this record was already in the database, false if it was
@@ -458,16 +556,33 @@ chassis_build_encaps(struct ovsdb_idl_txn *ovnsb_idl_txn,
 static bool
 chassis_get_record(struct ovsdb_idl_txn *ovnsb_idl_txn,
                    struct ovsdb_idl_index *sbrec_chassis_by_name,
+                   const struct sbrec_chassis_table *chassis_table,
+                   const struct ovs_chassis_cfg *ovs_cfg,
                    const char *chassis_id,
                    const struct sbrec_chassis **chassis_rec)
 {
-    const struct sbrec_chassis *chassis =
-        chassis = chassis_lookup_by_name(sbrec_chassis_by_name, chassis_id);
+    const struct sbrec_chassis *chassis = NULL;
 
-    if (!chassis && ovnsb_idl_txn) {
-        /* Create the chassis record. */
+    if (chassis_info_id_inited(&chassis_state)) {
+        chassis = chassis_lookup_by_name(sbrec_chassis_by_name,
+                                         chassis_info_id(&chassis_state));
+        if (!chassis) {
+            VLOG_DBG("Could not find Chassis, will check if the id changed: "
+                     "stored (%s) ovs (%s)",
+                     chassis_info_id(&chassis_state), chassis_id);
+        }
+    }
+
+    if (!chassis) {
+        chassis = chassis_get_stale_record(chassis_table, ovs_cfg, chassis_id);
+    }
+
+    if (!chassis) {
+        /* Recreate the chassis record. */
         VLOG_DBG("Could not find Chassis, will create it: %s", chassis_id);
-        *chassis_rec = sbrec_chassis_insert(ovnsb_idl_txn);
+        if (ovnsb_idl_txn) {
+            *chassis_rec = sbrec_chassis_insert(ovnsb_idl_txn);
+        }
         return false;
     }
 
@@ -502,6 +617,7 @@ chassis_update(const struct sbrec_chassis *chassis_rec,
                                      ovs_cfg->cms_options,
                                      ovs_cfg->monitor_all,
                                      ovs_cfg->chassis_macs,
+                                     ovs_cfg->enable_lflow_cache,
                                      &ovs_cfg->iface_types,
                                      ovs_cfg->is_interconn,
                                      chassis_rec)) {
@@ -514,6 +630,7 @@ chassis_update(const struct sbrec_chassis *chassis_rec,
                                    ovs_cfg->monitor_all,
                                    ovs_cfg->chassis_macs,
                                    ds_cstr_ro(&ovs_cfg->iface_types),
+                                   ovs_cfg->enable_lflow_cache,
                                    ovs_cfg->is_interconn);
         sbrec_chassis_verify_other_config(chassis_rec);
         sbrec_chassis_set_other_config(chassis_rec, &other_config);
@@ -533,6 +650,7 @@ chassis_update(const struct sbrec_chassis *chassis_rec,
                                 &ovs_cfg->encap_ip_set, ovs_cfg->encap_csum,
                                 chassis_rec);
     if (!tunnels_changed) {
+        chassis_update_encaps(chassis_rec);
         return updated;
     }
 
@@ -548,17 +666,94 @@ chassis_update(const struct sbrec_chassis *chassis_rec,
     return true;
 }
 
+/*
+ * Returns a pointer to a chassis_private record from 'chassis_pvt_table' that
+ * matches the chassis record.
+ */
+static const struct sbrec_chassis_private *
+chassis_private_get_stale_record(
+    const struct sbrec_chassis_private_table *chassis_pvt_table,
+    const struct sbrec_chassis *chassis)
+{
+    const struct sbrec_chassis_private *chassis_pvt_rec;
+
+    SBREC_CHASSIS_PRIVATE_TABLE_FOR_EACH (chassis_pvt_rec, chassis_pvt_table) {
+        if (chassis_pvt_rec->chassis == chassis) {
+            return chassis_pvt_rec;
+        }
+    }
+
+    return NULL;
+}
+
+/* If this is a chassis_private config update after we initialized the record
+ * once then we should always be able to find it with the ID we saved in
+ * chassis_state.
+ * Otherwise (i.e., first time we created the chassis record or if the
+ * system-id changed) then we check if there's a stale record from a previous
+ * controller run that didn't end gracefully and reuse it. If not then we
+ * create a new record.
+ *
+ * Returns the local chassis record.
+ */
+static const struct sbrec_chassis_private *
+chassis_private_get_record(
+    struct ovsdb_idl_txn *ovnsb_idl_txn,
+    struct ovsdb_idl_index *sbrec_chassis_pvt_by_name,
+    const struct sbrec_chassis_private_table *chassis_pvt_table,
+    const struct sbrec_chassis *chassis)
+{
+    const struct sbrec_chassis_private *chassis_p = NULL;
+
+    if (chassis_info_id_inited(&chassis_state)) {
+        chassis_p =
+            chassis_private_lookup_by_name(sbrec_chassis_pvt_by_name,
+                                           chassis_info_id(&chassis_state));
+    }
+
+    if (!chassis_p) {
+        chassis_p = chassis_private_get_stale_record(chassis_pvt_table,
+                                                     chassis);
+    }
+
+    if (!chassis_p && ovnsb_idl_txn) {
+        return sbrec_chassis_private_insert(ovnsb_idl_txn);
+    }
+
+    return chassis_p;
+}
+
+static void
+chassis_private_update(const struct sbrec_chassis_private *chassis_pvt,
+                       const struct sbrec_chassis *chassis,
+                       const char *chassis_id)
+{
+    if (!chassis_pvt->name || strcmp(chassis_pvt->name, chassis_id)) {
+        sbrec_chassis_private_set_name(chassis_pvt, chassis_id);
+    }
+
+    if (chassis_pvt->chassis != chassis) {
+        sbrec_chassis_private_set_chassis(chassis_pvt, chassis);
+    }
+}
+
 /* Returns this chassis's Chassis record, if it is available. */
 const struct sbrec_chassis *
 chassis_run(struct ovsdb_idl_txn *ovnsb_idl_txn,
             struct ovsdb_idl_index *sbrec_chassis_by_name,
+            struct ovsdb_idl_index *sbrec_chassis_private_by_name,
             const struct ovsrec_open_vswitch_table *ovs_table,
+            const struct sbrec_chassis_table *chassis_table,
+            const struct sbrec_chassis_private_table *chassis_pvt_table,
             const char *chassis_id,
             const struct ovsrec_bridge *br_int,
-            const struct sset *transport_zones)
+            const struct sset *transport_zones,
+            const struct sbrec_chassis_private **chassis_private)
 {
     struct ovs_chassis_cfg ovs_cfg;
 
+    *chassis_private = NULL;
+
     /* Get the chassis config from the ovs table. */
     ovs_chassis_cfg_init(&ovs_cfg);
     if (!chassis_parse_ovs_config(ovs_table, br_int, &ovs_cfg)) {
@@ -567,7 +762,8 @@ chassis_run(struct ovsdb_idl_txn *ovnsb_idl_txn,
 
     const struct sbrec_chassis *chassis_rec = NULL;
     bool existed = chassis_get_record(ovnsb_idl_txn, sbrec_chassis_by_name,
-                                      chassis_id, &chassis_rec);
+                                      chassis_table, &ovs_cfg, chassis_id,
+                                      &chassis_rec);
 
     /* If we found (or created) a record, update it with the correct config
      * and store the current chassis_id for fast lookup in case it gets
@@ -583,6 +779,17 @@ chassis_run(struct ovsdb_idl_txn *ovnsb_idl_txn,
                                       !existed ? "registering" : "updating",
                                       chassis_id);
         }
+
+        *chassis_private =
+            chassis_private_get_record(ovnsb_idl_txn,
+                                       sbrec_chassis_private_by_name,
+                                       chassis_pvt_table, chassis_rec);
+
+        if (*chassis_private) {
+            chassis_private_update(*chassis_private, chassis_rec, chassis_id);
+        }
+
+        chassis_info_set_id(&chassis_state, chassis_id);
     }
 
     ovs_chassis_cfg_destroy(&ovs_cfg);
@@ -638,16 +845,36 @@ chassis_get_mac(const struct sbrec_chassis *chassis_rec,
  * required. */
 bool
 chassis_cleanup(struct ovsdb_idl_txn *ovnsb_idl_txn,
-                const struct sbrec_chassis *chassis_rec)
+                const struct sbrec_chassis *chassis_rec,
+                const struct sbrec_chassis_private *chassis_private_rec)
 {
-    if (!chassis_rec) {
+    if (!chassis_rec && !chassis_private_rec) {
         return true;
     }
     if (ovnsb_idl_txn) {
         ovsdb_idl_txn_add_comment(ovnsb_idl_txn,
                                   "ovn-controller: unregistering chassis '%s'",
-                                  chassis_rec->name);
-        sbrec_chassis_delete(chassis_rec);
+                                  chassis_rec ? chassis_rec->name
+                                  : chassis_private_rec->name);
+        if (chassis_rec) {
+            sbrec_chassis_delete(chassis_rec);
+        }
+        if (chassis_private_rec) {
+            sbrec_chassis_private_delete(chassis_private_rec);
+        }
     }
     return false;
 }
+
+/*
+ * Returns the last initialized chassis-id.
+ */
+const char *
+chassis_get_id(void)
+{
+    if (chassis_info_id_inited(&chassis_state)) {
+        return chassis_info_id(&chassis_state);
+    }
+
+    return NULL;
+}
