@@ -29,6 +29,10 @@ const struct ovn_field ovn_fields[OVN_FIELD_N_IDS] = {
         OVN_ICMP4_FRAG_MTU,
         "icmp4.frag_mtu",
         2, 16,
+    }, {
+        OVN_ICMP6_FRAG_MTU,
+        "icmp6.frag_mtu",
+        4, 32,
     },
 };
 
@@ -119,10 +123,19 @@ ovn_init_symtab(struct shash *symtab)
                              flags_str);
 
     /* Connection tracking state. */
-    expr_symtab_add_field(symtab, "ct_mark", MFF_CT_MARK, NULL, false);
+    expr_symtab_add_field_scoped(symtab, "ct_mark", MFF_CT_MARK, NULL, false,
+                                 WR_CT_COMMIT);
 
-    expr_symtab_add_field(symtab, "ct_label", MFF_CT_LABEL, NULL, false);
-    expr_symtab_add_subfield(symtab, "ct_label.blocked", NULL, "ct_label[0]");
+    expr_symtab_add_field_scoped(symtab, "ct_label", MFF_CT_LABEL, NULL,
+                                 false, WR_CT_COMMIT);
+    expr_symtab_add_subfield_scoped(symtab, "ct_label.blocked", NULL,
+                                    "ct_label[0]", WR_CT_COMMIT);
+    expr_symtab_add_subfield_scoped(symtab, "ct_label.natted", NULL,
+                                    "ct_label[1]", WR_CT_COMMIT);
+    expr_symtab_add_subfield_scoped(symtab, "ct_label.ecmp_reply_eth", NULL,
+                                    "ct_label[32..79]", WR_CT_COMMIT);
+    expr_symtab_add_subfield_scoped(symtab, "ct_label.ecmp_reply_port", NULL,
+                                    "ct_label[80..95]", WR_CT_COMMIT);
 
     expr_symtab_add_field(symtab, "ct_state", MFF_CT_STATE, NULL, false);
 
@@ -254,7 +267,10 @@ ovn_init_symtab(struct shash *symtab)
     expr_symtab_add_field(symtab, "sctp.src", MFF_SCTP_SRC, "sctp", false);
     expr_symtab_add_field(symtab, "sctp.dst", MFF_SCTP_DST, "sctp", false);
 
+    expr_symtab_add_field(symtab, "pkt.mark", MFF_PKT_MARK, NULL, false);
+
     expr_symtab_add_ovn_field(symtab, "icmp4.frag_mtu", OVN_ICMP4_FRAG_MTU);
+    expr_symtab_add_ovn_field(symtab, "icmp6.frag_mtu", OVN_ICMP6_FRAG_MTU);
 }
 
 const char *
