@@ -109,14 +109,13 @@ main(int argc, char *argv[])
     sbctl_cmd_init();
 
     /* Check if options are set via env var. */
-    char **argv_ = ovs_cmdl_env_parse_all(&argc, argv,
-                                          getenv("OVN_SBCTL_OPTIONS"));
+    argv = ovs_cmdl_env_parse_all(&argc, argv, getenv("OVN_SBCTL_OPTIONS"));
 
     /* Parse command line. */
-    char *args = process_escape_args(argv_);
+    char *args = process_escape_args(argv);
     shash_init(&local_options);
-    parse_options(argc, argv_, &local_options);
-    char *error = ctl_parse_commands(argc - optind, argv_ + optind,
+    parse_options(argc, argv, &local_options);
+    char *error = ctl_parse_commands(argc - optind, argv + optind,
                                      &local_options, &commands, &n_commands);
     if (error) {
         ctl_fatal("%s", error);
@@ -150,7 +149,8 @@ main(int argc, char *argv[])
         if (seqno != ovsdb_idl_get_seqno(idl)) {
             seqno = ovsdb_idl_get_seqno(idl);
             if (do_sbctl(args, commands, n_commands, idl)) {
-                break;
+                free(args);
+                exit(EXIT_SUCCESS);
             }
         }
 
@@ -159,13 +159,6 @@ main(int argc, char *argv[])
             poll_block();
         }
     }
-
-    for (int i = 0; i < argc; i++) {
-        free(argv_[i]);
-    }
-    free(argv_);
-    free(args);
-    exit(EXIT_SUCCESS);
 }
 
 static void
@@ -1398,6 +1391,9 @@ cmd_set_ssl(struct ctl_context *ctx)
 static const struct ctl_table_class tables[SBREC_N_TABLES] = {
     [SBREC_TABLE_CHASSIS].row_ids[0] = {&sbrec_chassis_col_name, NULL, NULL},
 
+    [SBREC_TABLE_CHASSIS_PRIVATE].row_ids[0]
+    = {&sbrec_chassis_private_col_name, NULL, NULL},
+
     [SBREC_TABLE_DATAPATH_BINDING].row_ids
      = {{&sbrec_datapath_binding_col_external_ids, "name", NULL},
         {&sbrec_datapath_binding_col_external_ids, "name2", NULL},
