@@ -476,7 +476,6 @@ static struct shash port_groups;
 static struct hmap dhcp_opts;   /* Contains "struct gen_opts_map"s. */
 static struct hmap dhcpv6_opts; /* Contains "struct gen_opts_map"s. */
 static struct hmap nd_ra_opts; /* Contains "struct gen_opts_map"s. */
-static struct controller_event_options event_opts;
 
 static struct ovntrace_datapath *
 ovntrace_datapath_find_by_sb_uuid(const struct uuid *sb_uuid)
@@ -904,7 +903,6 @@ read_flows(void)
             .dhcp_opts = &dhcp_opts,
             .dhcpv6_opts = &dhcpv6_opts,
             .nd_ra_opts = &nd_ra_opts,
-            .controller_event_opts = &event_opts,
             .pipeline = (!strcmp(sblf->pipeline, "ingress")
                          ? OVNACT_P_INGRESS
                          : OVNACT_P_EGRESS),
@@ -933,8 +931,10 @@ read_flows(void)
             continue;
         }
         if (match) {
-            match = expr_simplify(match, ovntrace_is_chassis_resident,
-                                  NULL);
+            match = expr_simplify(match);
+            match = expr_evaluate_condition(match,
+                                            ovntrace_is_chassis_resident,
+                                            NULL, NULL);
         }
 
         struct ovntrace_flow *flow = xzalloc(sizeof *flow);
@@ -984,8 +984,6 @@ read_gen_opts(void)
 
     hmap_init(&nd_ra_opts);
     nd_ra_opts_init(&nd_ra_opts);
-
-    controller_event_opts_init(&event_opts);
 }
 
 static void
@@ -2123,7 +2121,12 @@ execute_ovnfield_load(const struct ovnact_load *load,
                              ntohs(load->imm.value.be16_int));
         break;
     }
-
+    case OVN_ICMP6_FRAG_MTU: {
+        ovntrace_node_append(super, OVNTRACE_NODE_MODIFY,
+                             "icmp6.frag_mtu = %u",
+                             ntohs(load->imm.value.be16_int));
+        break;
+    }
     case OVN_FIELD_N_IDS:
     default:
         OVS_NOT_REACHED();
@@ -2323,6 +2326,11 @@ trace_actions(const struct ovnact *ovnacts, size_t ovnacts_len,
                           super);
             break;
 
+        case OVNACT_ICMP6_ERROR:
+            execute_icmp6(ovnact_get_ICMP6_ERROR(a), dp, uflow, table_id,
+                          pipeline, super);
+            break;
+
         case OVNACT_IGMP:
             /* Nothing to do for tracing. */
             break;
